@@ -3,8 +3,8 @@
 These imports allow the user to skip the file names in their import.
 """
 
+from .tc_base import TcWidget
 from .tc_widgets import (
-    TcWidget,
     TcLabel,
     TcPushButton,
     TcLineEdit,
