@@ -7,10 +7,8 @@ The `@pyqtSlot()` is Qt decorator. In many cases it is not essential, but
 it's good practice to add it anyway.
 """
 
-from typing import Optional, Any, Tuple, Union, Callable, List
-from abc import ABC, ABCMeta, abstractmethod
+from typing import Optional, Any, Union, List
 import os
-import time
 from PyQt5.QtWidgets import (
     QMainWindow,
     QWidget,
@@ -27,20 +25,14 @@ from PyQt5.QtWidgets import (
     QCheckBox,
     QSlider,
 )
-from PyQt5.QtCore import pyqtSlot, Qt, QTimer
+from PyQt5.QtCore import pyqtSlot, Qt
 from PyQt5.QtGui import QIcon
+from .tc_base import TcWidget, PACKAGE_DIR
 from .custom_widgets import GraphWidget
 
 from ..twincat.symbols import Symbol
 
 
-PACKAGE_DIR = os.path.abspath(os.path.dirname(__file__) + "/../..")
-
-
-class QABCMeta(ABCMeta, type(QWidget)):
-    """Create a meta class that combines ABC and the Qt meta class."""
-
-
 class Color:  # pylint: disable=too-few-public-methods
     """Collection of useful colours."""
 
@@ -48,242 +40,6 @@ class Color:  # pylint: disable=too-few-public-methods
     EDITING = "#FFFFAA"  # Yellow-ish
 
 
-class TcWidget(ABC, metaclass=QABCMeta):
-    """Abstract class, to be multi-inherited together with a Qt item.
-
-    There are different event types, which determine how and when new remote
-    values are retrieved:
-
-        * `EVENT_NOTIFICATION`: An ADS notification is created, resulting in a
-          callback on a remote value change. Suitable for rarely changing values or
-          when a very quick response is needed.
-          ADS notifications have some overhead. No more than 200 symbol
-          notifications should exist at the same time.
-        * `EVENT_TIMER`: New values are read at a fixed interval. Useful when
-          remote values change often but no instant response is needed. This
-          method has very little overhead.
-        * `EVENT_NONE`: No attempts are made to update according to remote values.
-
-    You can override the defaults for your entire project by changing the
-    ``TcWidget.DEFAULT_EVENT_TYPE`` and ``TcWidget.DEFAULT_UPDATE_FREQ``
-    class properties. Make sure to update these before any widgets are
-    created and they will have new standard values.
-
-    :cvar DEFAULT_EVENT_TYPE: (default: EVENT_NOTIFICATION)
-    :cvar DEFAULT_UPDATE_FREQ: (default: 10.0 Hz)
-    """
-
-    EVENT_NOTIFICATION = "notification"
-    EVENT_TIMER = "timer"
-    EVENT_NONE = "none"
-
-    DEFAULT_EVENT_TYPE = EVENT_NOTIFICATION
-    DEFAULT_UPDATE_FREQ = 10.0  # Hz
-
-    def __init__(self, *args, **kwargs):
-        """
-
-        It is important to call this init() as late as possible from a
-        subclass! The order should be:
-
-           1. Subclass specific stuff (e.g. number formatting)
-           2. Call to super().__init__(...) - Now the QWidget stuff has been
-              made ready
-           3. QWidget related stuff
-
-        Note: these methods do not affect when or how a value is _written_ to the ADS
-        pool.
-
-        :param args:
-        :param kwargs: See list below - kwargs are passed along to
-            `connect_symbol` too
-
-        :kwargs:
-            * `symbol`: ``Symbol`` to link to
-                        (i.e. to read from and/or write to)
-            * `format`: Formatter symbol, e.g. '%.1f' or '%d' or callable
-                        ('%.3f' by default, ignored when not relevant)
-                        Callable must have a single argument
-            * `event_type`:     Possible values are EVENT_* constants
-                                (default: ``DEFAULT_EVENT_TYPE``)
-            * `update_freq`:    Frequency (Hz) for timed update
-                                (for EVENT_TIMER only, default: ``DEFAULT_UPDATE_FREQ``)
-            * `greyed`:         When true, the widget is visibly disabled
-                                When false, the widget is shown normally even when
-                                disconnected (default: `true`)
-
-        """
-
-        self._symbol: Optional[Symbol] = kwargs.pop("symbol", None)
-        # Included None fallback
-
-        self.value_format: Union[str, Callable[[Any], str]] = kwargs.pop(
-            "format", "%.3f"
-        )
-
-        self.event_type = kwargs.pop("event_type", self.DEFAULT_EVENT_TYPE)
-
-        self.update_freq = kwargs.pop("update_freq", self.DEFAULT_UPDATE_FREQ)
-
-        self.greyed = kwargs.pop("greyed", True)
-
-        self._handles: Optional[Tuple[int, int]] = None
-        # Handles to the notification specific for this widget
-
-        self._skip_event: bool = False  # If true, QWidget events should not result in
-        # a change of the ADS symbol
-
-        self._last_update: Optional[float] = None  # Timestamp of the last successful
-        # twincat_send(), used to prevent an event loop
-
-        self._timer: Optional[QTimer] = None  # For timed event loop
-        self._last_value: Optional[Any] = None  # For timed event loop
-
-        # Disable widget
-        if self.greyed:
-            self.setDisabled(True)
-
-        if self._symbol:
-            self.connect_symbol(self._symbol, **kwargs)
-            # Connect already if passed
-
-    def connect_symbol(self, new_symbol: Optional[Symbol] = None, **kwargs):
-        """Connect a symbol (copy is left as property).
-
-        By default a device callback is created with an on-change event
-        from TwinCAT.
-        Old callbacks are deleted first. Pass None to only clear callbacks.
-        The notification handles are stored locally.
-        Extend (= override but call the parent first) this method to
-        configure more of the widget, useful if e.g. widget callbacks depend
-        on the symbol.
-
-        :param new_symbol: Symbol to link to (set None to only clear the
-            previous)
-        :param kwargs: See list below - Keyword arguments are passed along as
-            device notification settings too
-
-        :kwargs:
-            * `event_type`:     See :class:`TcWidget`
-            * `update_freq`:    See :class:`TcWidget`
-        """
-
-        if self._handles is not None and self._symbol is not None:
-            self._symbol.del_device_notification(self._handles)
-            # In case previous callback existed, clear it
-            self._handles = None
-
-        self._timer = None  # Destroy old timer
-
-        self._symbol = new_symbol
-
-        if "event_type" in kwargs:
-            self.event_type = kwargs.pop("event_type")
-        if "update_freq" in kwargs:
-            self.update_freq = kwargs.pop("update_freq")
-
-        if new_symbol is None:
-            if self.greyed:
-                self.setDisabled(True)
-        else:
-            connected = True  # New connected state
-            if self.event_type == self.EVENT_NOTIFICATION:
-                self._handles = self._symbol.add_device_notification(
-                    self.twincat_receive_wrapper, **kwargs
-                )
-                # It seems a notification is always fired on creation, so we don't
-                # need to call it now
-                if self._handles is None:  # In case the callback quietly failed
-                    connected = False
-            elif self.event_type == self.EVENT_TIMER:
-                self._timer = QTimer()
-                self._timer.timeout.connect(self.on_timeout)
-                m_sec = int(1000 / self.update_freq)
-                self._timer.start(m_sec)
-            elif self.event_type == self.EVENT_NONE:
-                pass  # Nothing to be done
-            else:
-                ValueError("Unrecognized event type: " + self.event_type)
-
-            if self.greyed and connected:
-                self.setDisabled(False)
-
-    def twincat_receive_wrapper(self, value):
-        """Intermediate twincat_receive callback to prevent event loops."""
-
-        # If incoming value equals old buffered value
-        if value == self._symbol.value and self._last_update is not None:
-
-            elapsed_ms = (time.time() - self._last_update) * 1000
-            if elapsed_ms < 50:
-                # If within 50 ms of the last update, discard this notification
-                # This is typically a callback after a twincat_send()
-                return
-
-        self.twincat_receive(value)
-
-    @pyqtSlot()
-    def on_timeout(self):
-        """Callback for the event timer."""
-        # We cannot use the buffered _symbol.value, since it could have been updated
-        # outside this widget instance.
-        new_val = self._symbol.read()
-        if new_val != self._last_value:
-            # Trigger incoming-value callback
-            self.twincat_receive(new_val)
-            self._last_value = new_val
-
-    @abstractmethod
-    def twincat_receive(self, value):
-        """Callback attached to the TwinCAT symbol.
-
-        Note: changing a state of a widget (e.g. checkbox being checked through
-        `setChecked(True)`) will typically fire the on-change events again. So be
-        careful to prevent an event loop when updating a widget based on a remote
-        change: a change could result in a state change, which could result in a
-        remote change, etc.
-
-        :param value: New remote value
-        """
-
-    def twincat_send(self, value: Any):
-        """Set value in symbol (and send to TwinCAT).
-
-        Method is safe: if symbol is not connected, nothing will happen.
-        """
-        self._last_update = time.time()  # Current floating point timestamp
-
-        if self._symbol is not None:  # Safe for unconnected symbols
-            self._symbol.set(value)
-
-    def format(self, value: Any) -> str:
-        """ "Use the stored formatting to created a formatted text.
-
-        In case the format specifier is a string and the new value is a list,
-        element-wise string formatting will be concatenated automatically.
-        """
-        if isinstance(self.value_format, str):
-            if isinstance(value, list):
-                elements = [self.value_format % item for item in value]
-                return ", ".join(elements)
-
-            return self.value_format % value
-
-        if callable(self.value_format):
-            return self.value_format(value)
-
-        raise NotImplementedError(
-            "The format `{}` could not be processed".format(self.value_format)
-        )
-
-    def __del__(self):
-        """Destructor."""
-
-        if self._symbol is not None:
-            # Element is about to become extinct, so clear callbacks
-            self._symbol.clear_device_notifications()
-
-
 class TcLabel(QLabel, TcWidget):
     """Label that shows a value."""
 
@@ -532,7 +288,7 @@ class TcCheckBox(QCheckBox, TcWidget):
         """
 
         self.value_checked = kwargs.pop("value_checked", 1)
-        self.value_unchecked = kwargs.pop("value_checked", 0)
+        self.value_unchecked = kwargs.pop("value_unchecked", 0)
 
         super().__init__(*args, **kwargs)
 
@@ -781,11 +537,13 @@ class TcGraph(GraphWidget, TcWidget):
 
         super().connect_symbol(new_symbol, **kwargs)
 
-    @pyqtSlot()
-    def on_timeout(self):
-        """Callback for the event timer (override)."""
+    def on_mass_timeout(self):
+        """Callback for the event timer (override).
 
-        values = [symbol.read() for symbol in self._symbol]
+        This assumes the remote read was already performed!
+        """
+
+        values = [symbol._value for symbol in self._symbol]
 
         value_x = self.symbol_x.read() if self.symbol_x is not None else None
 
@@ -794,7 +552,7 @@ class TcGraph(GraphWidget, TcWidget):
     def twincat_receive(self, value):
         """Abstract implementation.
 
-        All useful code is in on_timeout() instead.
+        All useful code is in on_mass_timeout() instead.
         """
         pass
 
